const fs = require('fs')
const transform = require('css-to-react-native').default
var csstree = require('css-tree')
// import {Dimensions} from 'react-native'
var path = require('path')

function queryImport(styles) {
    if (styles.indexOf('@import') < 0) return []
    const arr = []
    const ast = csstree.parse(styles);
    csstree.walk(ast, {
        enter: function (node) {
            if (node.type === 'Atrule' && node.name === 'import') {
                const pat = node.prelude.children.head.data.value
                arr.push(pat.substring(1, pat.length - 1))
            }
        }
    })
    return arr
}

function handleImportStyle(styles, initPath, mayRM) {
    if (!!mayRM) {
        styles = styles.replace(mayRM, '')
    }
    if (styles.indexOf('@import') < 0) return styles
    const ast = csstree.parse(styles);
    csstree.walk(ast, {
        enter: function (node) {
            if (node.type === 'Atrule') {
                if (node.name === 'import') {
                    const pat = node.prelude.children.head.data.value
                    if (initPath && typeof pat === 'string') {
                        const realP = pat.substring(1, pat.length - 1)
                        initPath = initPath.substring(0, initPath.lastIndexOf('/'))
                        const resPath = path.resolve(initPath, realP)
                        const text = fs.readFileSync(resPath).toString()
                        const re = new RegExp("[\\s]*@import[\\s]+\"" + realP + "\";", "g");
                        //may have loop import
                        styles = styles.replace(re, handleImportStyle(text, resPath, re))
                    }
                }
            }
        }
    })
    return styles
}


function createStylefromCode(styles, abspath) {
    const arr = queryImport(styles)
    const baseRes = handleImportStyle(styles, abspath)
    // console.log(baseRes)
    const obj = {}
    const ast = csstree.parse(baseRes);
    csstree.walk(ast, {
        enter: function (node) {
            if (node.type === 'Rule' && node.prelude && node.prelude.type === 'SelectorList') {
                const clzName = []
                node.prelude.children.forEach(e => {
                    //we may handle each class selector here
                    let ans = selectorBlockHandler(e)
                    if (ans) {
                        //to save before value
                        obj[ans] = obj[ans] || {}
                        //save ans to log which to give value
                        clzName.push(ans)
                    }
                })
                const styleArray = []
                node.block.children.forEach(e => {
                    const styleItem = styleBlockHandler(e)
                    styleItem.length > 0 && styleArray.push(styleItem)
                })
                clzName.forEach(e => {
                    Object.assign(obj[e], transform(styleArray))
                })
            }
        },
    })
    return {styles: obj, imports: arr}
}

function createStyleDictionary(styles) {
    let total = {}
    for (let k in styles) {
        const val = styles[k]
        k = sortSeq(k)
        const lastEle = getLastEle(k)
        if (!total.hasOwnProperty(lastEle)) {
            total[lastEle] = createItem(k, val)[lastEle]
        } else {
            merge(createItem(k, styles[k]), total)
        }
    }
    return total
}

function countCharFromStr(str, ch) {
    let strArr = [...str];
    let count = 0;
    for (let k in strArr) {
        if (strArr[k] === ch) {
            count++
        }
    }
    return count
}

const sortSeq = (str) => {
    return str.split(" ").filter(e => !!e && e.trim().length > 0).map(e => {
        if (countCharFromStr(e, '.') > 1) {
            return '.' + e.split(".").filter(e => !!e && e.trim().length > 0).sort().join(".")
        } else return e
    }).join(" ")
}

const createItem = (name, val) => {
    let nameArr = name.split(" ").filter(e => !!e && e.trim().length > 0).reverse()
    const map = {}
    let refMap = map
    let len = nameArr.length
    for (let ind in nameArr) {
        if (ind < len) {
            refMap[nameArr[ind]] = {}
            refMap = refMap[nameArr[ind]]
        }
    }
    //结束标识
    refMap["_#_"] = val
    return map
}

//linka为单链，并入linkb
const merge = (linka, linkb) => {
    let refa = linka
    let refb = linkb
    //a为单链，所以每次遍历，key其实都只有一个,b可以有很多
    while (Object.keys(refa).length) {
        for (let key in refa) {
            if (refb.hasOwnProperty(key)) {
                if (key === '_#_') {
                    refb[key] = Object.assign({}, refa[key], refb[key])
                    return linkb
                }
                refa = refa[key]
                refb = refb[key]
                break
            } else {
                //找到不同，进行合并
                refb[key] = refa[key]
                return linkb
            }
        }
    }
}
const getLastEle = (str) => {
    return str.split(" ").filter(e => e.length > 0).pop()
}


function serailize(obj, str = "", whetherQuoto = true) {
    let tmp = '{';
    for (let k in obj) {
        if (obj.hasOwnProperty(k)) {
            if (whetherQuoto)
                tmp += "\"" + k + "\":"
            else tmp += k + ":"
            if (typeof obj[k] === 'object' && !Array.isArray(obj[k])) {
                if (k === '_#_')
                    tmp += serailize(obj[k], str, false)[0]
                else tmp += serailize(obj[k], str, true)[0]
            } else {
                if (isQuoto(obj[k]))
                    tmp += obj[k] + ","
                else if (whetherQuoto || isNaN(obj[k]) && !Array.isArray(obj[k]))
                    tmp += "\"" + obj[k] + "\"" + ","
                //may be array here
                else {
                    if (Array.isArray(obj[k])) {
                        tmp += serailizeArray(obj[k]) + ","
                    } else
                        tmp += obj[k] + ","
                }
            }
        }
    }
    tmp += "},"
    str = tmp + str
    return [tmp, str]
}

function isQuoto(str) {
    if (typeof str !== 'string') return false
    if (!!str && str.length > 1) {
        if (str[0] === '\"' && str[str.length - 1] === '\"') {
            return true
        }
        if (str[0] === '\'' && str[str.length - 1] === '\'') {
            return true
        }
    }
    return false
}

function serailizeArray(arr, str = "") {
    if (!Array.isArray(arr)) return serailize(arr)
    let ans = '['
    for (let k in arr) {
        if (Array.isArray(arr[k]))
            ans += serailizeArray(arr[k], str)
        else if (typeof arr[k] === 'object')
            ans += serailize(arr[k], "", false)[1].replace(/,}/g, '}')
        else if (typeof arr[k] === 'string')
            ans += arr[k]
    }
    ans = ans.substring(0, ans.length - 1)
    ans += ']'
    str = ans + str
    return str
}

function convertStylesToRNCSS(styles) {
    const cssMap = createStyleDictionary(styles)
    const str = serailize(cssMap)[1].replace(/,}/g, '}')
    return str.substring(0, str.length - 1)
}

function selectorBlockHandler(element) {
    if (element.type === 'Selector') {
        const name = []
        let discard = false
        element.children.forEach(e => {
            switch (e.type) {
                case "ClassSelector":
                    name.push("." + e.name);
                    break;
                case "Combinator":
                    name.push(e.name);
                    discard = true
                    return null
                case "TypeSelector":
                    name.push(e.name);
                    break;
                case "IdSelector":
                    name.push("#" + e.name);
                    discard = true
                    return null
                case "WhiteSpace":
                    name.push(" ");
                    break
                default: {
                    // console.warn('unknown selector type:', e)
                    discard = true
                    return null
                }
            }
        })
        if (!discard)
            return name.join("")
        else return null
    }
}

function paddingElementForTransform(proper, arr) {
    if (proper === 'font-family') {
        return [proper, arr.pop()]
    }
    if (proper === 'background') {
        return ['background-color', arr.join(" ")]
    }
    if (proper === 'border-bottom' || proper === 'border-top' || proper === 'border-right' || proper === 'border-left') {
        // console.warn("not support property",proper)
        return []
    }
    if (proper === 'content' && arr.length === 1)
        return [proper, arr[0]]
    return [proper, arr.join(" ")]
}
//TODO let transform give unit
function queryUnit(item) {
    return ""
}

function styleBlockHandler(element) {
    const {property, important, value} = element
    let result = []
    const styleContent = value && value.children ? value.children : []
    const defaultUnit = "px"
    styleContent.forEach(e => {
        switch (e.type) {
            case "Identifier":
                result.push(e.name);
                break
            case "Dimension":
                if (e.unit.trim() === 'rpx')
                    result.push(e.value / 2 + defaultUnit);
                else result.push(e.value + defaultUnit);
                break
            case 'Function': {
                let name = e.name + "("
                e.children.forEach(e => name += e.value + queryUnit(e.name))
                result.push(name + ")")
                break
            }
            case "HexColor": {
                result.push("#" + e.value)
                break
            }
            case "String": {
                const str = e.value
                result.push(str)
                break
            }
            case "Number": {
                result.push(e.value)
                break
            }
            case "Percentage": {
                result.push(e.value + "%")
                break
            }
            case "WhiteSpace": {
                break
            }
            case "Operator": {
                break
            }
            case "Url": {
                break
            }
            default:
                // console.log("unknown,", property, e)
                break
        }
    })
    if (result.length > 0)
        return paddingElementForTransform(property, result)
    return []
}

module.exports = {
    createStylefromCode: createStylefromCode,
    convertStylesToRNCSS: convertStylesToRNCSS
}